@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* job.c: job control */
-/* (C) 2007-2016 magicant */
+/* (C) 2007-2017 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -80,6 +80,8 @@ static bool jobs_builtin_print_job(size_t jobnumber,
 	bool runningonly, bool stoppedonly);
 static int continue_job(size_t jobnumber, job_T *job, bool fg)
     __attribute__((nonnull));
+static int wait_for_job_by_jobspec(const wchar_t *jobspec)
+    __attribute__((nonnull));
 static bool wait_builtin_has_job(bool jobcontrol);
 
 
@@ -319,9 +321,14 @@ void apply_curstop(void)
 size_t job_count(void)
 {
     size_t count = 0;
-    for (size_t i = 0; i < joblist.length; i++)
-	if (joblist.contents[i] != NULL)
-	    count++;
+    for (size_t i = 0; i < joblist.length; i++) {
+	const job_T *job = joblist.contents[i];
+	if (job == NULL)
+	    continue;
+	if (job->j_status == JS_DONE && !job->j_statuschanged)
+	    continue; // Ignore finished jobs that have already been reported.
+	count++;
+    }
     return count;
 }
 
@@ -724,6 +731,18 @@ char *get_job_status_string(const job_T *job, bool *needfree)
     assert(false);
 }
 
+/* Returns true iff there is any job whose status has been changed but not yet
+ * reported. If this function returns false, `print_job_status_all' is nop. */
+bool any_job_status_has_changed(void)
+{
+    for (size_t i = 1; i < joblist.length; i++) {
+	const job_T *job = get_job(i);
+	if (job != NULL && !job->j_nonotify && job->j_statuschanged)
+	    return true;
+    }
+    return false;
+}
+
 /* Prints the status of the specified job.
  * If `remove_done' is true, finished jobs are removed from the job list after
  * the status is printed.
@@ -742,7 +761,7 @@ int print_job_status(size_t jobnumber,
     if (job == NULL || job->j_nonotify)
 	return result;
     if (changedonly && !job->j_statuschanged)
-	goto done;
+	return result;
 
     char current;
     if      (jobnumber == current_jobnumber)  current = '+';
@@ -798,7 +817,6 @@ int print_job_status(size_t jobnumber,
 	}
     }
     job->j_statuschanged = false;
-done:
     if (remove_done && job->j_status == JS_DONE)
 	remove_job(jobnumber);
 
@@ -1271,47 +1289,16 @@ int wait_builtin(int argc, void **argv)
 	}
     }
 
-    job_T *job;
     if (xoptind < argc) {
 	/* wait for the specified jobs */
-	do {
-	    const wchar_t *jobspec = ARGV(xoptind);
-	    size_t jobnumber;
-	    if (jobspec[0] == L'%') {
-		jobnumber = get_jobnumber_from_name(&jobspec[1]);
-	    } else {
-		long pid;
-		if (!xwcstol(jobspec, 10, &pid) || pid < 0) {
-		    xerror(0, Ngt("`%ls' is not a valid job specification"),
-			    jobspec);
-		    continue;
-		}
-		jobnumber = get_jobnumber_from_pid(pid);
+	for (; xoptind < argc; xoptind++) {
+	    int jobstatus = wait_for_job_by_jobspec(ARGV(xoptind));
+	    if (jobstatus < 0) {
+		status = -jobstatus;
+		break;
 	    }
-	    if (jobnumber >= joblist.length) {
-		xerror(0, Ngt("job specification `%ls' is ambiguous"),
-			ARGV(xoptind));
-	    } else if (jobnumber == 0
-		    || (job = joblist.contents[jobnumber]) == NULL
-		    || job->j_legacy) {
-		status = Exit_NOTFOUND;
-	    } else {
-		status = wait_for_job(jobnumber, jobcontrol, jobcontrol, true);
-		if (status == 0) {
-		    status = calc_status_of_job(job);
-		} else {
-		    assert(TERMSIGOFFSET >= 128);
-		    status += TERMSIGOFFSET;
-		    break;
-		}
-		if (job->j_status != JS_RUNNING) {
-		    if (jobcontrol && is_interactive_now && !posixly_correct)
-			print_job_status(jobnumber, false, false, true, stdout);
-		    else if (job->j_status == JS_DONE)
-			remove_job(jobnumber);
-		}
-	    }
-	} while (++xoptind < argc);
+	    status = jobstatus;
+	}
     } else {
 	/* wait for all jobs */
 	while (wait_builtin_has_job(jobcontrol)) {
@@ -1324,28 +1311,67 @@ int wait_builtin(int argc, void **argv)
 	}
     }
 
-    if (status != 0)
-	return status;
     if (yash_error_message_count != 0)
 	return Exit_FAILURE;
-    return Exit_SUCCESS;
+    return status;
+}
+
+/* Finds a job specified by the argument and waits for it.
+ * Returns a negated exit status if interrupted. */
+int wait_for_job_by_jobspec(const wchar_t *jobspec)
+{
+    size_t jobnumber;
+    if (jobspec[0] == L'%') {
+	jobnumber = get_jobnumber_from_name(&jobspec[1]);
+    } else {
+	long pid;
+	if (!xwcstol(jobspec, 10, &pid) || pid < 0) {
+	    xerror(0, Ngt("`%ls' is not a valid job specification"), jobspec);
+	    return Exit_FAILURE;
+	}
+	jobnumber = get_jobnumber_from_pid(pid);
+    }
+    if (jobnumber >= joblist.length) {
+	xerror(0, Ngt("job specification `%ls' is ambiguous"), jobspec);
+	return Exit_FAILURE;
+    }
+
+    job_T *job;
+    if (jobnumber == 0
+	    || (job = joblist.contents[jobnumber]) == NULL
+	    || job->j_legacy)
+	return Exit_NOTFOUND;
+
+    int signal = wait_for_job(jobnumber,
+	    doing_job_control_now, doing_job_control_now, true);
+    if (signal != 0) {
+	assert(TERMSIGOFFSET >= 128);
+	return -(signal + TERMSIGOFFSET);
+    }
+
+    int status = calc_status_of_job(job);
+    if (job->j_status != JS_RUNNING) {
+	if (doing_job_control_now && is_interactive_now && !posixly_correct)
+	    print_job_status(jobnumber, false, false, true, stdout);
+	else if (job->j_status == JS_DONE)
+	    remove_job(jobnumber);
+    }
+    return status;
 }
 
 /* Checks if the shell has any job to wait for. */
 bool wait_builtin_has_job(bool jobcontrol)
 {
     /* print/remove already-finished jobs */
-    if (jobcontrol && is_interactive_now && !posixly_correct) {
-	for (size_t i = 1; i < joblist.length; i++)
-	    print_job_status(i, true, false, true, stdout);
-    } else {
-	for (size_t i = 1; i < joblist.length; i++) {
-	    job_T *job = joblist.contents[i];
-	    if (job != NULL && (job->j_legacy || job->j_status == JS_DONE))
-		remove_job(i);
-	}
+    for (size_t i = 1; i < joblist.length; i++) {
+	job_T *job = joblist.contents[i];
+	if (jobcontrol && is_interactive_now && !posixly_correct)
+	    print_job_status(i, true, false, false, stdout);
+	if (job != NULL && (job->j_legacy || job->j_status == JS_DONE))
+	    remove_job(i);
     }
 
+    /* see if we have jobs to wait for. */
     for (size_t i = 1; i < joblist.length; i++) {
 	job_T *job = joblist.contents[i];
 	if (job != NULL && (!jobcontrol || job->j_status == JS_RUNNING))
