@@ -993,6 +993,9 @@ pipeline_T *parse_pipeline(parsestate_T *ps)
     if (has_token(ps, L"!")) {
 	neg = true;
 	ps->index += 1;
+	if (posixly_correct && ps->src.contents[ps->index] == L'(')
+	    serror(ps, Ngt("ksh-like extended glob pattern `!(...)' "
+			"is not supported"));
 	do {
 	    skip_blanks_and_comment(ps);
 	    c = parse_commands_in_pipeline(ps);
@@ -2153,7 +2156,7 @@ command_T *parse_for(parsestate_T *ps)
     }
     result->c_forname = name;
 
-    skip_to_next_token(ps);
+    bool on_next_line = skip_to_next_token(ps);
     ensure_buffer(ps, 3);
     if (has_token(ps, L"in")) {
 	redir_T *redirs = NULL;
@@ -2170,9 +2173,8 @@ command_T *parse_for(parsestate_T *ps)
 	result->c_forwords = NULL;
 	if (ps->src.contents[ps->index] == L';') {
 	    ps->index++;
-	    if (posixly_correct)
-		serror(ps, Ngt("`;' is not allowed "
-			    "just after the identifier in a for loop"));
+	    if (on_next_line)
+		serror(ps, Ngt("`;' cannot appear on a new line"));
 	}
     }
     skip_to_next_token(ps);
